@@ -4,18 +4,177 @@ DBI::Changes - List of significant changes to the DBI
 
 =cut
 
-Need to add docs for DbTypeSubclass (ala DBIx::AnyDBD)
-Move TIEHASH etc to XS (and pureperl)
+=head1 CHANGES in DBI 1.43 (svn rev 373),    28th June 2004
 
-$dbh->{Statement} can be wrong because fetch doesn't update value
-maybe imp_dbh holds imp_sth (or inner handle) of last sth method
-called (if not DESTROY) and sth outer DESTROY clears it (to reduce ref count)
-Then $dbh->{LastSth} would work (returning outer handle if valid).
-Then $dbh->{Statement} would be the same as $dbh->{LastSth}->{Statement}
-Also $dbh->{ParamValues} would be the same as $dbh->{LastSth}->{ParamValues}.
-Add more macro hooks to Driver.xst: ping, quote etc.
+  Fixed connect() and connect_cached() RaiseError/PrintError
+    which would sometimes show "(no error string)" as the error.
+  Fixed compiler warning thanks to Paul Marquess.
+  Fixed "trace level set to" trace message thanks to H.Merijn Brand.
+  Fixed DBD::DBM $dbh->{dbm_tables}->{...} to be keyed by the
+    table name not the file name thanks to Jeff Zucker.
+  Fixed last_insert_id(...) thanks to Rudy Lippan.
+  Fixed propagation of scalar/list context into proxied methods.
+  Fixed DBI::Profile::DESTROY to not alter $@.
+  Fixed DBI::ProfileDumper new() docs thanks to Michael Schwern.
+  Fixed _load_class to propagate $@ thanks to Drew Taylor.
+  Fixed compile warnings on Win32 thanks to Robert Baron.
+  Fixed problem building with recent versions of MakeMaker.
+  Fixed DBD::Sponge not to generate warning with threads.
+  Fixed DBI_AUTOPROXY to work more than once thanks to Steven Hirsch.
 
-=head1 CHANGES
+  Changed TraceLevel 1 to not show recursive/nested calls.
+  Changed getting or setting an invalid attribute to no longer be
+    a fatal error but generate a warning instead.
+  Changed selectall_arrayref() to call finish() if
+    $attr->{MaxRows} is defined.
+  Changed all tests to use Test::More and enhanced the tests thanks
+    to Stevan Little and Andy Lester. See http://qa.perl.org/phalanx/
+  Changed Test::More minimum prerequisite version to 0.40 (2001).
+  Change DBI::Profile header to include timestamp.
+
+  Added DBI->parse_dsn($dsn) method.
+  Added warning if build directory path contains whitespace.
+  Added docs for parse_trace_flags() and parse_trace_flag().
+  Removed "may change" warnings from the docs for table_info(),
+    primary_key_info(), and foreign_key_info() methods.
+
+=head1 CHANGES in DBI 1.42 (svn rev 222),    12th March 2004
+
+  Fixed $sth->{NUM_OF_FIELDS} of non-executed statement handle
+    to be undef as per the docs (it was 0).
+  Fixed t/41prof_dump.t to work with perl5.9.1.
+  Fixed DBD_ATTRIB_DELETE macro thanks to Marco Paskamp.
+  Fixed DBI::PurePerl looks_like_number() and $DBI::rows.
+  Fixed ref($h)->can("foo") to not croak.
+
+  Changed attributes (NAME, TYPE etc) of non-executed statement
+    handle to be undef instead of triggering an error.
+  Changed ShowErrorStatement to apply to more $dbh methods.
+  Changed DBI_TRACE env var so just does this at load time:
+    DBI->trace(split '=', $ENV{DBI_TRACE}, 2);
+  Improved "invalid number of parameters" error message.
+  Added DBI::common as base class for DBI::db, DBD::st etc.
+  Moved methods common to all handles into DBI::common.
+
+  Major tracing enhancement:
+
+  Added $h->parse_trace_flags("foo|SQL|7") to map a group of
+    trace flags into the corresponding trace flag bits.
+  Added automatic calling of parse_trace_flags() if
+    setting the trace level to a non-numeric value:
+    $h->{TraceLevel}="foo|SQL|7"; $h->trace("foo|SQL|7");
+    DBI->connect("dbi:Driver(TraceLevel=SQL|foo):...", ...);
+    Currently no trace flags have been defined.
+  Added to, and reworked, the trace documentation.
+  Added dbivport.h for driver authors to use.
+
+  Major driver additions that Jeff Zucker and I have been working on:
+
+  Added DBI::SQL::Nano a 'smaller than micro' SQL parser
+    with an SQL::Statement compatible API. If SQL::Statement
+    is installed then DBI::SQL::Nano becomes an empty subclass
+    of SQL::Statement, unless the DBI_SQL_NANO env var is true.
+  Added DBD::File, modified to use DBI::SQL::Nano.
+  Added DBD::DBM, an SQL interface to DBM files using DBD::File.
+
+  Documentation changes:
+
+  Corrected typos in docs thanks to Steffen Goeldner.
+  Corrected execute_for_fetch example thanks to Dean Arnold.
+
+=head1 CHANGES in DBI 1.41 (svn rev 130),    22nd February 2004
+
+  Fixed execute_for_array() so tuple_status parameter is optional
+    as per docs, thanks to Ed Avis.
+  Fixed execute_for_array() docs to say that it returns undef if
+    any of the execute() calls fail.
+  Fixed take_imp_data() test on m68k reported by Christian Hammers.
+  Fixed write_typeinfo_pm inconsistencies in DBI::DBD::Metadata
+    thanks to Andy Hassall.
+  Fixed $h->{TraceLevel} to not return DBI->trace trace level
+    which it used to if DBI->trace trace level was higher.
+
+  Changed set_err() to append to errstr, with a leading "\n" if it's
+    not empty, so that multiple error/warning messages are recorded.
+  Changed trace to limit elements dumped when an array reference is
+    returned from a method to the max(40, $DBI::neat_maxlen/10)
+    so that fetchall_arrayref(), for example, doesn't flood the trace.
+  Changed trace level to be a four bit integer (levels 0 thru 15)
+    and a set of topic flags (no topics have been assigned yet).
+  Changed column_info() to check argument count.
+  Extended bind_param() TYPE attribute specification to imply
+    standard formating of value, eg SQL_DATE implies 'YYYY-MM-DD'.
+
+  Added way for drivers to indicate 'success with info' or 'warning'
+    by setting err to "0" for warning and "" for information.
+    Both values are false and so don't trigger RaiseError etc.
+    Thanks to Steffen Goeldner for the original idea.
+  Added $h->{HandleSetErr} = sub { ... } to be called at the
+    point that an error, warn, or info state is recorded.
+    The code can alter the err, errstr, and state values
+    (e.g., to promote an error to a warning, or the reverse).
+  Added $h->{PrintWarn} attribute to enable printing of warnings
+    recorded by the driver. Defaults to same value as $^W (perl -w).
+  Added $h->{ErrCount} attribute, incremented whenever an error is
+    recorded by the driver via set_err().
+  Added $h->{Executed} attribute, set if do()/execute() called.
+  Added \%attr parameter to foreign_key_info() method.
+  Added ref count of inner handle to "DESTROY ignored for outer" msg.
+  Added Win32 build config checks to DBI::DBD thanks to Andy Hassall.
+  Added bind_col to Driver.xst so drivers can define their own.
+  Added TYPE attribute to bind_col and specified the expected
+    driver behaviour.
+
+  Major update to signal handling docs thanks to Lincoln Baxter.
+  Corrected dbiproxy usage doc thanks to Christian Hammers.
+  Corrected type_info_all index hash docs thanks to Steffen Goeldner.
+  Corrected type_info COLUMN_SIZE to chars not bytes thanks to Dean Arnold.
+  Corrected get_info() docs to include details of DBI::Const::GetInfoType.
+  Clarified that $sth->{PRECISION} is OCTET_LENGTH for char types.
+
+=head2 Changes in DBI 1.40,    7th January 2004
+
+  Fixed handling of CachedKids when DESTROYing threaded handles.
+  Fixed sql_user_name() in DBI::DBD::Metadata (used by write_getinfo_pm)
+    to use $dbh->{Username}. Driver authors please update your code.
+
+  Changed connect_cached() when running under Apache::DBI
+    to route calls to Apache::DBI::connect().
+
+  Added CLONE() to DBD::Sponge and DBD::ExampleP.
+  Added warning when starting a new thread about any loaded driver
+    which does not have a CLONE() function.
+  Added new prepare_cache($sql, \%attr, 3) option to manage Active handles.
+  Added SCALE and NULLABLE support to DBD::Sponge.
+  Added missing execute() in fetchall_hashref docs thanks to Iain Truskett.
+  Added a CONTRIBUTING section to the docs with notes on creating patches.
+
+=head2 Changes in DBI 1.39,    27th November 2003
+
+  Fixed STORE to not clear error during nested DBI call, again/better,
+    thanks to Tony Bowden for the report and helpful test case.
+  Fixed DBI dispatch to not try to use AUTOLOAD for driver methods unless
+    the method has been declared (as methods should be when using AUTOLOAD).
+    This fixes a problem when the Attribute::Handlers module is loaded.
+  Fixed cwd check code to use $Config{path_sep} thanks to Steve Hay.
+  Fixed unqualified croak() calls thanks to Steffen Goeldner.
+  Fixed DBD::ExampleP TYPE and PRECISION attributes thanks to Tom Lowery.
+  Fixed tracing of methods that only get traced at high trace levels.
+
+  The level 1 trace no longer includes nested method calls so it generally
+    just shows the methods the application explicitly calls.
+  Added line to trace log (level>=4) when err/errstr is cleared.
+  Updated docs for InactiveDestroy and point out where and when the
+    trace includes the process id.
+  Update DBI::DBD docs thanks to Steffen Goeldner.
+  Removed docs saying that the DBI->data_sources method could be
+    passed a $dbh. The $dbh->data_sources method should be used instead.
+  Added link to 'DBI recipes' thanks to Giuseppe Maxia:
+    http://gmax.oltrelinux.com/dbirecipes.html (note that this
+    is not an endorsement that the recipies are 'optimal')
+
+  Note: There is a bug in perl 5.8.2 when configured with threads
+  and debugging enabled (bug #24463) which causes a DBI test to fail.
 
 =head2 Changes in DBI 1.38,    21th August 2003
 
